@@ -1,5 +1,5 @@
 #
-# $Id: Search.pm,v 9163f2f2b320 2015/01/04 12:34:23 gomor $
+# $Id: Search.pm,v b5b16a8eab97 2015/10/12 19:03:34 gomor $
 #
 # brik::search Brik
 #
@@ -11,7 +11,7 @@ use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 9163f2f2b320 $',
+      revision => '$Revision: b5b16a8eab97 $',
       tags => [ qw(unstable main brik search) ],
       commands => {
          all => [ ],
@@ -20,6 +20,7 @@ sub brik_properties {
          not_tag => [ qw(Tag) ],
          used => [ ],
          not_used => [ ],
+         show_require_modules => [ ],
       },
    };
 }
@@ -176,6 +177,41 @@ sub not_used {
    return $self->not_tag('used');
 }
 
+sub show_require_modules {
+   my $self = shift;
+
+   my $context = $self->context;
+   my $available = $context->available;
+
+   # Don't show require for Metabrik::Core
+   my $core = {
+      'shell::script',
+      'shell::rc',
+      'shell::history',
+      'shell::command',
+      'brik::search',
+      'perl::module',
+      'core::context',
+      'core::log',
+      'core::shell',
+      'core::global',
+   };
+
+   my %require_modules = ();
+   for my $brik (keys %$available) {
+      next if (exists($core->{$brik}));
+      if ($available->{$brik}->can('brik_properties')) {
+         my $modules = $available->{$brik}->brik_properties->{require_modules};
+         for my $module (keys %$modules) {
+            next if $module =~ /^Metabrik/;
+            $require_modules{$module} = $brik;
+         }
+      }
+   }
+
+   return [ sort { $a cmp $b } keys %require_modules ];
+}
+
 1;
 
 __END__
