@@ -1,17 +1,17 @@
 #
-# $Id: Context.pm,v d32c8bd9203c 2015/03/09 19:52:04 gomor $
+# $Id: Context.pm,v 519956f054f8 2015/10/17 10:48:12 gomor $
 #
 package Metabrik::Core::Context;
 use strict;
 use warnings;
 
-our $VERSION = '1.08';
+our $VERSION = '1.10';
 
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: d32c8bd9203c $',
+      revision => '$Revision: 519956f054f8 $',
       tags => [ qw(core context main) ],
       attributes => {
          _lp => [ qw(INTERNAL) ],
@@ -206,7 +206,9 @@ sub call {
       my @list = caller();
       my $file = $list[1];
       my $line = $list[2];
-      #return $self->log->error("call: $@ (source file [$file] at line [$line])");
+      if ($self->debug) {
+         return $self->log->error("call: $@ (source file [$file] at line [$line])");
+      }
       return $self->log->error("call: $@");
    }
 
@@ -407,9 +409,48 @@ sub use {
 sub reuse {
    my $self = shift;
 
-   my $reused = Module::Reload->check;
-   if ($reused) {
-      $self->log->info("reuse: some modules were reused");
+   my %stat = ();
+   my @reloaded = ();
+   # Taken from Module::Reload
+   for my $entry (map { [ $_, $INC{$_} ] } keys %INC) {
+      my ($module, $file) = @$entry;
+
+      if ($file eq $INC{"Module/Reload.pm"}) {
+         next;   # Too confusing
+      }
+
+      local $^W = 0;  # Disable 'use warnings';
+      my $mtime = (stat $file)[9];
+      if (! defined($stat{$file})) {
+         $stat{$file} = $^T;
+      }
+
+      next unless defined($mtime);
+
+      if ($mtime > $stat{$file}) {
+         delete $INC{$module};
+         eval { 
+            $SIG{__WARN__} = sub {};
+            require $module;
+         };
+         if ($@) {
+            chomp($@);
+            if ($self->debug) {
+               $self->log->error("reuse: reloading module [$module] failed: [$@]");
+            }
+            else {
+               $self->log->error("reuse: reloading module [$module] failed");
+            }
+         }
+         else {
+            push @reloaded, $module;
+         }
+      }
+      $stat{$file} = $mtime;
+   }
+
+   for (@reloaded) {
+      $self->log->verbose("reuse: module [$_] successfully reloaded");
    }
 
    return 1;
@@ -566,8 +607,16 @@ sub set {
          die("$MSG\n");
       }
 
-      if ($__ctx_value =~ /^(\$.*)$/) {
-         $__ctx_value = eval("\$CON->_lp->{context}->{_}->{'$1'}");
+      if ($__ctx_value =~ /^\$\w+/) {
+         eval {
+            $__ctx_value = $CON->_lp->do($__ctx_value);
+         };
+         if ($@) {
+            chomp($@);
+            $ERR = 1;
+            my $MSG = "set: Brik [$__ctx_brik] has invalid argument [$__ctx_value]";
+            die("$MSG\n");
+         }
       }
 
       $CON->{used}->{$__ctx_brik}->$__ctx_attribute($__ctx_value);
@@ -666,8 +715,31 @@ sub run {
       }
 
       for (@__ctx_args) {
-         if (/^(\$.*)$/) {
-            $_ = eval("\$CON->_lp->{context}->{_}->{'$1'}");
+         # Support variable lookups like '$array' as an Argument
+         # Example: run <Brik> <Command> $Arg1 Arg2
+         if (/^\$\w+/) {
+            eval {
+               $_ = $CON->_lp->do($_);
+            };
+            if ($@) {
+               chomp($@);
+               $ERR = 1;
+               my $MSG = "run: Brik [$__ctx_brik] has invalid argument [$_]";
+               die("$MSG\n");
+            }
+         }
+         # Support passing ARRAYs or HASHs or Perl code as an Argument
+         # Example: run <Brik> <Command> "[ qw(a b c) ]"
+         elsif (/^\[.*\]$/ || /^\{.*\}$/) {
+            eval {
+               $_ = $CON->_lp->do($_);
+            };
+            if ($@) {
+               chomp($@);
+               $ERR = 1;
+               my $MSG = "run: Brik [$__ctx_brik] has invalid argument [$_]";
+               die("$MSG\n");
+            }
          }
       }
 
@@ -710,7 +782,7 @@ sub save_state {
       }
 
       my $__ctx_state;
-      my $__ctx_attributes = $CON->{used}->{$__ctx_brik}->brik_properties->{attributes} || {};
+      my $__ctx_attributes = $CON->{used}->{$__ctx_brik}->brik_attributes || {};
       for my $__ctx_this (keys %$__ctx_attributes) {
          $__ctx_state->{$__ctx_this} = $CON->{used}->{$__ctx_brik}->$__ctx_this;
       }
