@@ -1,5 +1,5 @@
 #
-# $Id: Shell.pm,v 1ebb8741c3c2 2015/03/18 06:42:55 gomor $
+# $Id: Shell.pm,v 69c83035069a 2015/10/16 17:49:51 gomor $
 #
 # core::shell Brik
 #
@@ -7,7 +7,7 @@ package Metabrik::Core::Shell;
 use strict;
 use warnings;
 
-our $VERSION = '1.08';
+our $VERSION = '1.10';
 
 use base qw(Term::Shell Metabrik);
 
@@ -15,17 +15,32 @@ use IO::All;
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1ebb8741c3c2 $',
+      revision => '$Revision: 69c83035069a $',
       tags => [ qw(core main shell) ],
       attributes => {
          echo => [ qw(0|1) ],
-         help_show_brik_commands => [ qw(0|1) ],
-         help_show_brik_attributes => [ qw(0|1) ],
-         comp_show_brik_attributes => [ qw(0|1) ],
-         comp_show_brik_commands => [ qw(0|1) ],
-         get_show_brik_attributes => [ qw(0|1) ],
-         help_show_inherited => [ qw(0|1) ],
-         comp_show_inherited => [ qw(0|1) ],
+         help_show_base_attributes => [ qw(0|1) ],
+         help_show_base_commands => [ qw(0|1) ],
+         help_show_base_all => [ qw(0|1) ],   # Both Attributes and Commands
+         help_show_inherited_attributes => [ qw(0|1) ],
+         help_show_inherited_commands => [ qw(0|1) ],
+         help_show_inherited_all => [ qw(0|1) ],  # Both Attributes and Commands
+         help_show_all => [ qw(0|1) ],  # Both Attributes and Commands for base and inherited
+         comp_show_base_attributes => [ qw(0|1) ],
+         comp_show_base_commands => [ qw(0|1) ],
+         comp_show_base_all => [ qw(0|1) ],
+         comp_show_inherited_attributes => [ qw(0|1) ],
+         comp_show_inherited_commands => [ qw(0|1) ],
+         comp_show_inherited_all => [ qw(0|1) ],  # Both Attributes and Commands
+         comp_show_all => [ qw(0|1) ],  # Both Attributes and Commands for base and inherited
+         show_base_attributes => [ qw(0|1) ],
+         show_base_commands => [ qw(0|1) ],
+         show_base_all => [ qw(0|1) ],
+         show_inherited_attributes => [ qw(0|1) ],
+         show_inherited_commands => [ qw(0|1) ],
+         show_inherited_all => [ qw(0|1) ],
+         show_all => [ qw(0|1) ],  # Both Attributes and Commands for base and inherited
+         aliases_completion => [ qw(0|1) ], # Complete aliases to show original Command
          # These are used by Term::Shell
          #path_home => [ qw(directory) ],
          #path_cwd => [ qw(directory) ],
@@ -35,17 +50,38 @@ sub brik_properties {
       },
       attributes_default => {
          echo => 1,
-         help_show_brik_commands => 0,
-         help_show_brik_attributes => 0,
-         comp_show_brik_attributes => 0,
-         comp_show_brik_commands => 0,
-         get_show_brik_attributes => 0,
-         help_show_inherited => 0,
-         comp_show_inherited => 0,
+         help_show_base_attributes => 0,
+         help_show_base_commands => 0,
+         help_show_base_all => 0,
+         help_show_inherited_attributes => 0,
+         help_show_inherited_commands => 0,
+         help_show_inherited_all => 0,
+         help_show_all => 0,
+         comp_show_base_attributes => 0,
+         comp_show_base_commands => 0,
+         comp_show_base_all => 0,
+         comp_show_inherited_attributes => 0,
+         comp_show_inherited_commands => 0,
+         comp_show_inherited_all => 0,
+         comp_show_all => 0,
+         show_base_attributes => 0,
+         show_base_commands => 0,
+         show_base_all => 0,
+         show_inherited_attributes => 0,
+         show_inherited_commands => 0,
+         show_inherited_all => 0,
+         show_all => 0,
+         aliases_completion => 1,
       },
       commands => {
          splash => [ ],
          pwd => [ ],
+         get_available_help => [ ],
+         get_help_attributes => [ qw(Brik) ],
+         get_help_commands => [ qw(Brik) ],
+         get_comp_attributes => [ qw(Brik) ],
+         get_comp_commands => [ qw(Brik) ],
+         # Term::Shell stuff
          cmd => [ qw(Cmd) ],
          cmdloop => [ ],
          run_use => [ qw(Brik) ],
@@ -176,6 +212,24 @@ sub pwd {
    return $self->{path_cwd};
 }
 
+sub get_available_help {
+   my $self = shift;
+
+   my @used = sort { $a cmp $b } keys %{$self->context->used};
+   my @aliases = sort { $a cmp $b } keys %{$self->{_aliases}};
+   my @commands = sort { $a cmp $b } keys %{$self->brik_commands};
+
+   # Skip class functions
+   @commands = grep (!/^brik_/, @commands);
+
+   # Remove leading run_ string
+   for (@aliases, @commands) {
+      s/^run_//;
+   }
+
+   return { briks => \@used, aliases => \@aliases, commands => \@commands };
+}
+
 #
 # Term::Shell stuff
 #
@@ -193,7 +247,7 @@ sub AUTOLOAD {
 
    (my $command = $AUTOLOAD) =~ s/^Metabrik::Core::Shell:://;
 
-   $self->debug && $self->log->debug("command[$command] args[@args]");
+   $self->debug && $self->log->debug("AUTOLOAD: command[$command] args[@args]");
 
    #my $aliases = $self->_aliases;
    my $aliases = $self->{_aliases};
@@ -216,6 +270,46 @@ sub AUTOLOAD {
    return 1;
 }
 
+# We overwrite inherited Term::Shell rl_complete() sub to have control
+sub rl_complete {
+   my $self = shift;
+   my ($word, $line, $start) = @_;
+
+   my @comp = ();
+
+   # If it's a command, complete 'run_':
+   if ($start == 0 || substr($line, 0, $start) =~ /^\s*$/) {
+      $self->debug && $self->log->debug("rl_complete: word[$word] start[$start] line[$line]");
+      @comp = $self->complete('', $word, $line, $start);
+
+      $self->debug && $self->log->debug("rl_complete: comp[@comp]");
+
+      # If we found something and it's an alias, we complete with the original command
+      if (defined($comp[0]) && $self->comp_aliases && exists($self->{_aliases}{"run_".$comp[0]})) {
+         $self->debug && $self->log->debug("rl_complete: original[".$self->{_aliases}{"run_$comp[0]"}."]");
+
+         my @words = split(/\s+/, $self->{_aliases}{"run_$comp[0]"});
+         if (exists($self->{_aliases}{"run_$words[0]"})) {
+            @comp = $self->{_aliases}{"run_".$words[0]};
+         }
+         else {
+            @comp = $self->{_aliases}{"run_$comp[0]"};
+         }
+      }
+   }
+   # If it's a subcommand, send it to any custom completion function for the
+   # function:
+   else {
+      my $command = ($self->line_parsed($line))[0];
+      $self->debug && $self->log->debug("rl_complete: send to custom completion");
+      @comp = $self->complete($command, $word, $line, $start);
+   }
+
+   $self->debug && $self->log->debug("rl_complete: return comp[@comp]");
+
+   return @comp;
+}
+
 # Converts Windows path
 sub _convert_path {
    my ($path) = @_;
@@ -297,6 +391,10 @@ sub init {
    # See also Term::ReadLine LoadTermCap() and ornaments() subs.
    $self->term->ornaments('md,me');
 
+   # Force Commands to be entered entirely to avoid ambiguity.
+   # Example: type 'my' will result in excuting Perl code, and not 'mymeta-cpanfile'.
+   $self->{API}{match_uniq} = 0;
+
    return $self;
 }
 
@@ -330,13 +428,20 @@ sub cmd_to_code {
    my $self = shift;
    my ($line) = @_;
 
-   while ($line =~ /'\s*(?:use|set|get|run)\s.*?'/) {
-      #$self->log->info("cmd_to_code: before: [$line]");
-      $line =~ s/'\s*((?:use|set|get|run)\s.*?)\s*'/\$SHE->cmd("$1")/;
-      #$self->log->info("cmd_to_code: after: [$line]");
+   # Example: a run Command converted to Perl Code:
+   #  'run shell::command system "echo $_"';
+   #  $SHE->cmd("run shell::command system \"echo $_\"");
 
-      $self->debug && $self->log->debug("cmd_to_code: [$line]");
+   $self->debug && $self->log->debug("cmd_to_code: before: [$line]");
+   if ($line =~ /^\s*'\s*((?:use|set|get|run)\s.*?)\s*'\s*;?\s*$/) {
+      # We have to escape " chars
+      (my $new = $1) =~ s{"}{\\"}g;
+      # We have to escape $ variables too so they are completed
+      $new =~ s{\$}{\\\$}g;
+      $self->debug && $self->log->debug("cmd_to_code: new: [$new]");
+      $line = '$SHE->cmd("'.$new.'");';
    }
+   $self->debug && $self->log->debug("cmd_to_code: after: [$line]");
 
    return $line;
 }
@@ -361,12 +466,13 @@ sub process_line {
    # If a closure is open, we are in multiline mode
    if (! $self->cmd_is_complete($lines)) {
       # If it looks like a Metabrik command, we rewrite it to a Perl code string
+      # This is to support 'run <Brik> <Command> <Args>' within multiline.
       $lines->[-1] = $self->cmd_to_code($line);
       $self->_update_prompt('.. ');
       return 1;
    }
 
-   $self->debug && $self->log->debug("cmdloop: lines[@$lines]");
+   $self->debug && $self->log->debug("process_line: lines[@$lines]");
 
    $self->cmd(join('', @$lines));
 
@@ -434,6 +540,10 @@ sub run_exit {
 }
 
 sub comp_exit {
+   my $self = shift;
+
+   $self->debug && $self->log->debug("comp_exit: true");
+
    return ();
 }
 
@@ -445,13 +555,19 @@ sub run_alias {
    my $aliases = $self->{_aliases};
 
    if (! defined($alias)) {
-      for my $this (keys %$aliases) {
-         (my $alias = $this) =~ s/^run_//;
+      for my $this (sort { $a cmp $b } keys %$aliases) {
+         ($alias = $this) =~ s/^run_//;
          $self->log->info(sprintf("%-10s \"%s\"", $alias, $aliases->{$this}));
       }
 
       return 1;
    }
+   elsif (length($alias) && @cmd == 0) {
+      $alias =~ s/^run_//;
+      $self->log->info(sprintf("%-10s \"%s\"", $alias, $aliases->{"run_$alias"}));
+
+      return 1;
+   }
 
    $aliases->{"run_$alias"} = join(' ', @cmd);
    #$self->_aliases($aliases);
@@ -463,6 +579,10 @@ sub run_alias {
 }
 
 sub comp_alias {
+   my $self = shift;
+
+   $self->debug && $self->log->debug("comp_alias: true");
+
    return ();
 }
 
@@ -478,12 +598,14 @@ sub run_cd {
       if (! -d $dir) {
          return $self->log->error("cd: $dir: can't cd to this");
       }
-      chdir($dir);
+      chdir($dir)
+         or return $self->log->error("cd: $dir: $!");
       $self->_update_path_cwd;
    }
    else {
       #chdir($self->path_home);
-      chdir($self->{path_home});
+      chdir($self->{path_home})
+         or return $self->log->error("cd: $dir: $!");
       $self->_update_path_cwd;
    }
 
@@ -496,6 +618,8 @@ sub comp_cd {
    my $self = shift;
    my ($word, $line, $start) = @_;
 
+   $self->debug && $self->log->debug("comp_cd: true");
+
    return $self->catch_comp_sub($word, $start, $line);
 }
 
@@ -541,6 +665,8 @@ sub comp_code {
    my $self = shift;
    my ($word, $line, $start) = @_;
 
+   $self->debug && $self->log->debug("comp_code: true");
+
    return $self->catch_comp_sub($word, $start, $line);
 }
 
@@ -574,6 +700,8 @@ sub comp_use {
    my $self = shift;
    my ($word, $line, $start) = @_;
 
+   $self->debug && $self->log->debug("comp_use: true");
+
    my $context = $self->context;
 
    my @words = $self->line_parsed($line);
@@ -602,49 +730,266 @@ sub comp_use {
    return @comp;
 }
 
-sub run_help {
+sub get_help_attributes {
    my $self = shift;
    my ($brik) = @_;
 
+   if (! defined($brik)) {
+      return $self->log->error($self->brik_help_run('get_help_attributes'));
+   }
+
    my $context = $self->context;
 
-   if (! defined($brik)) {
-      return $self->SUPER::run_help;
+   if (! $context->is_used($brik)) {
+      return {};
    }
-   else {
-      if ($context->is_used($brik)) {
-         my $used_brik = $context->used->{$brik};
 
-         my $attributes = $used_brik->brik_attributes;
-         my $commands = $used_brik->brik_commands;
+   my $used = $context->used;
 
-         if (! $self->help_show_inherited) {
-            $attributes = $used_brik->brik_properties->{attributes};
-            $commands = $used_brik->brik_properties->{commands};
-         }
+   my $attributes = $used->{$brik}->brik_own_attributes;
 
-         my $brik_attributes = Metabrik->brik_properties->{attributes};
-         my $brik_commands = Metabrik->brik_properties->{commands};
+   my $base_attributes = {};
+   if ($self->help_show_base_attributes || $self->help_show_base_all
+   ||  $self->show_base_attributes || $self->show_base_all
+   ||  $self->help_show_all || $self->show_all) {
+      $base_attributes = $used->{$brik}->brik_base_attributes;
+   }
 
-         for my $attribute (keys %$attributes) {
-            if (! $context->get('core::shell', 'help_show_brik_attributes')) {
-               next if exists($brik_attributes->{$attribute});
-            }
+   my $inherited_attributes = {};
+   if ($self->help_show_inherited_attributes || $self->help_show_inherited_all
+   ||  $self->show_inherited_attributes || $self->show_inherited_all
+   ||  $self->help_show_all || $self->show_all) {
+      $inherited_attributes = $used->{$brik}->brik_inherited_attributes;
+   }
+
+   for my $attribute (keys %$base_attributes) {
+      $attributes->{$attribute} = $base_attributes->{$attribute};
+   }
+
+   for my $attribute (keys %$inherited_attributes) {
+      $attributes->{$attribute} = $inherited_attributes->{$attribute};
+   }
+
+   return $attributes;
+}
+
+sub get_help_commands {
+   my $self = shift;
+   my ($brik) = @_;
+
+   if (! defined($brik)) {
+      return $self->log->error($self->brik_help_run('get_help_commands'));
+   }
+
+   my $context = $self->context;
+
+   if (! $context->is_used($brik)) {
+      return {};
+   }
+
+   my $used = $context->used;
+
+   my $commands = $used->{$brik}->brik_own_commands;
+
+   my $base_commands = {};
+   if ($self->help_show_base_commands || $self->help_show_base_all
+   ||  $self->show_base_commands || $self->show_base_all
+   ||  $self->help_show_all || $self->show_all) {
+      $base_commands = $used->{$brik}->brik_base_commands;
+   }
+
+   my $inherited_commands = {};
+   if ($self->help_show_inherited_commands || $self->help_show_inherited_all
+   ||  $self->show_inherited_commands || $self->show_inherited_all
+   ||  $self->help_show_all || $self->show_all) {
+      $inherited_commands = $used->{$brik}->brik_inherited_commands;
+   }
+
+   for my $command (keys %$base_commands) {
+      $commands->{$command} = $base_commands->{$command};
+   }
+
+   for my $command (keys %$inherited_commands) {
+      $commands->{$command} = $inherited_commands->{$command};
+   }
+
+   return $commands;
+}
+
+sub get_comp_attributes {
+   my $self = shift;
+   my ($brik) = @_;
+
+   if (! defined($brik)) {
+      return $self->log->error($self->brik_help_run('get_comp_attributes'));
+   }
+
+   my $context = $self->context;
+
+   if (! $context->is_used($brik)) {
+      return {};
+   }
+
+   my $used = $context->used;
+
+   my $attributes = $used->{$brik}->brik_own_attributes;
+
+   my $base_attributes = {};
+   if ($self->comp_show_base_attributes || $self->comp_show_base_all
+   ||  $self->show_base_attributes || $self->show_base_all
+   ||  $self->comp_show_all || $self->show_all) {
+      $base_attributes = $used->{$brik}->brik_base_attributes;
+   }
+
+   my $inherited_attributes = {};
+   if ($self->comp_show_inherited_attributes || $self->comp_show_inherited_all
+   ||  $self->show_inherited_attributes || $self->show_inherited_all
+   ||  $self->comp_show_all || $self->show_all) {
+      $inherited_attributes = $used->{$brik}->brik_inherited_attributes;
+   }
+
+   for my $attribute (keys %$base_attributes) {
+      $attributes->{$attribute} = $base_attributes->{$attribute};
+   }
+
+   for my $attribute (keys %$inherited_attributes) {
+      $attributes->{$attribute} = $inherited_attributes->{$attribute};
+   }
+
+   return $attributes;
+}
+
+sub get_comp_commands {
+   my $self = shift;
+   my ($brik) = @_;
+
+   if (! defined($brik)) {
+      return $self->log->error($self->brik_help_run('get_comp_commands'));
+   }
+
+   my $context = $self->context;
+
+   if (! $context->is_used($brik)) {
+      return {};
+   }
+
+   my $used = $context->used;
+
+   my $commands = $used->{$brik}->brik_own_commands;
+
+   my $base_commands = {};
+   if ($self->comp_show_base_commands || $self->comp_show_base_all
+   ||  $self->show_base_commands || $self->show_base_all
+   ||  $self->comp_show_all || $self->show_all) {
+      $base_commands = $used->{$brik}->brik_base_commands;
+   }
+
+   my $inherited_commands = {};
+   if ($self->comp_show_inherited_commands || $self->comp_show_inherited_all
+   ||  $self->show_inherited_commands || $self->show_inherited_all
+   ||  $self->comp_show_all || $self->show_all) {
+      $inherited_commands = $used->{$brik}->brik_inherited_commands;
+   }
+
+   for my $command (keys %$base_commands) {
+      $commands->{$command} = $base_commands->{$command};
+   }
+
+   for my $command (keys %$inherited_commands) {
+      $commands->{$command} = $inherited_commands->{$command};
+   }
+
+   return $commands;
+}
+
+sub run_help {
+   my $self = shift;
+   my ($arg1, $arg2) = @_;
+
+   my $context = $self->context;
+
+   my $help = $self->get_available_help;
+   my %aliases = map { $_ => 1 } @{$help->{aliases}};
+   my %briks = map { $_ => 1 } @{$help->{briks}};
+   my %commands = map { $_ => 1 } @{$help->{commands}};
+
+   # Print the list of available Briks, Aliases, Commands
+   if (! defined($arg1)) {
+      $self->log->info("For more help, print help <Command>:");
+      $self->log->info(" ");
+
+      for my $this (sort { $a cmp $b } @{$help->{briks}}) {
+         $self->log->info("   $this - Brik");
+      }
+
+      for my $this (sort { $a cmp $b } @{$help->{aliases}}) {
+         $self->log->info("   $this - Alias");
+      }
+
+      for my $this (sort { $a cmp $b } @{$help->{commands}}) {
+         $self->log->info("   $this - Command");
+      }
+
+      return 1;
+   }
+   elsif (! defined($arg2)) {
+      # Help for a Brik
+      if ($context->is_used($arg1)) {
+         my $used_brik = $context->used->{$arg1};
+
+         my $attributes = $self->get_help_attributes($arg1);
+         my $commands = $self->get_help_commands($arg1);
+
+         for my $attribute (sort { $a cmp $b } keys %$attributes) {
             my $help = $used_brik->brik_help_set($attribute);
             $self->log->info($help) if defined($help);
          }
 
-         for my $command (keys %$commands) {
-            if (! $context->get('core::shell', 'help_show_brik_commands')) {
-               next if exists($brik_commands->{$command});
-            }
+         for my $command (sort { $a cmp $b } keys %$commands) {
             my $help = $used_brik->brik_help_run($command);
             $self->log->info($help) if defined($help);
          }
       }
+      # Help for a Command
+      elsif (exists($commands{$arg1})) {
+         for my $this (sort { $a cmp $b } keys %commands) {
+            return $self->log->info("$arg1 - core::shell Command, see 'help core::shell $arg1'");
+         }
+      }
+      # Help for an Alias
+      elsif (exists($aliases{$arg1})) {
+         return $self->log->info("$arg1 - no help for Aliases");
+      }
       else {
-         # We return to standard help() method
-         return $self->SUPER::run_help($brik);
+         return $self->log->info("Command [$arg1] not found");
+      }
+   }
+   # Both arg1 and arg2 are defined
+   else {
+      if (exists($briks{$arg1})) {
+         my $used_brik = $context->used->{$arg1};
+
+         my $attributes = $used_brik->brik_attributes;
+         my $commands = $used_brik->brik_commands;
+
+         my $base_attributes = $used_brik->brik_base_attributes;
+         my $base_commands = $used_brik->brik_base_commands;
+
+         my $help;
+         if (exists($attributes->{$arg2}) || exists($base_attributes->{$arg2})) {
+            $help = $used_brik->brik_help_set($arg2);
+         }
+         elsif (exists($commands->{$arg2}) || exists($base_commands->{$arg2})) {
+            $help = $used_brik->brik_help_run($arg2);
+         }
+         else {
+            $help = "Attribute or Command [$arg2] not found for Brik [$arg1]";
+         }
+
+         return $self->log->info($help);
+      }
+      else {
+         return $self->log->info("Command [$arg1] not found");
       }
    }
 
@@ -655,6 +1000,8 @@ sub comp_help {
    my $self = shift;
    my ($word, $line, $start) = @_;
 
+   $self->debug && $self->log->debug("comp_help: true");
+
    my @words = $self->line_parsed($line);
    my $count = scalar(@words);
 
@@ -664,16 +1011,22 @@ sub comp_help {
 
    my @comp = ();
 
-   # We want to find help for used briks by using completion
-   if (($count == 1)
-   ||  ($count == 2 && length($word) > 0)) {
-      my @handlers = keys %{$self->{handlers}};
-      my @used = keys %{$self->context->used};
-      for my $a (@handlers, @used) {
+   # We want to find help for used Briks/Aliases/Commands by using completion
+   if ($count == 1 || ($count == 2 && length($word) > 0)) {
+      my $help = $self->get_available_help;
+      # No need to complete aliases, there is no help
+      #for my $a (@{$help->{briks}}, @{$help->{aliases}}, @{$help->{commands}}) {
+      for my $a (@{$help->{briks}}, @{$help->{commands}}) {
          next unless length($a);
          push @comp, $a if $a =~ /^$word/;
       }
    }
+   # We want to complete entered Command and Attributes
+   else {
+      push @comp, $self->comp_run(@_);
+      push @comp, $self->comp_set(@_);
+      return @comp;
+   }
 
    return @comp;
 }
@@ -700,6 +1053,8 @@ sub comp_set {
    my $self = shift;
    my ($word, $line, $start) = @_;
 
+   $self->debug && $self->log->debug("comp_set: true");
+
    my $context = $self->context;
 
    # Completion is for used Briks only
@@ -736,17 +1091,9 @@ sub comp_set {
          }
       }
 
-      my $brik_attributes = Metabrik->brik_properties->{attributes};
-      my $attributes = $used->{$brik}->brik_attributes;
-
-      if (! $self->comp_show_inherited) {
-         $attributes = $used->{$brik}->brik_properties->{attributes};
-      }
+      my $attributes = $self->get_comp_attributes($brik);
 
       for my $attribute (keys %$attributes) {
-         if (! $context->get('core::shell', 'comp_show_brik_attributes')) {
-            next if exists($brik_attributes->{$attribute});
-         }
          push @comp, $attribute;
       }
    }
@@ -759,17 +1106,9 @@ sub comp_set {
          }
       }
 
-      my $brik_attributes = Metabrik->brik_properties->{attributes};
-      my $attributes = $used->{$brik}->brik_attributes;
-
-      if (! $self->comp_show_inherited) {
-         $attributes = $used->{$brik}->brik_properties->{attributes};
-      }
+      my $attributes = $self->get_comp_attributes($brik);
 
       for my $attribute (keys %$attributes) {
-         if (! $context->get('core::shell', 'comp_show_brik_attributes')) {
-            next if exists($brik_attributes->{$attribute});
-         }
          if ($attribute =~ /^$word/) {
             push @comp, $attribute;
          }
@@ -794,13 +1133,9 @@ sub run_get {
       my $used = $context->used or return;
 
       for my $brik (sort { $a cmp $b } keys %$used) {
-         my $attributes = $used->{$brik}->brik_attributes or next;
+         my $attributes = $self->get_help_attributes($brik);
+
          for my $attribute (sort { $a cmp $b } keys %$attributes) {
-            if (! $context->get('core::shell', 'get_show_brik_attributes')) {
-               if ($attribute =~ /^(?:shell|context|log|global|init_done|debug)$/) {
-                  next;
-               }
-            }
             $self->log->info("$brik $attribute ".$context->get($brik, $attribute));
          }
       }
@@ -813,17 +1148,10 @@ sub run_get {
          return $self->log->error("get: Brik [$brik] not used");
       }
 
-      my %printed = ();
-      my $attributes = $used->{$brik}->brik_attributes;
+      my $attributes = $self->get_help_attributes($brik);
+
       for my $attribute (sort { $a cmp $b } keys %$attributes) {
-         if (! $context->get('core::shell', 'get_show_brik_attributes')) {
-            if ($attribute =~ /^(?:shell|context|log|global|init_done|debug)$/) {
-               next;
-            }
-         }
-         my $print = "$brik $attribute ".$context->get($brik, $attribute);
-         $self->log->info($print) if ! exists($printed{$print});
-         $printed{$print}++;
+         $self->log->info("$brik $attribute ".$context->get($brik, $attribute));
       }
    }
    # get is called with is a Brik and an Attribute
@@ -847,6 +1175,8 @@ sub run_get {
 sub comp_get {
    my $self = shift;
 
+   $self->debug && $self->log->debug("comp_get: true");
+
    return $self->comp_set(@_);
 }
 
@@ -876,13 +1206,18 @@ sub comp_run {
    my $self = shift;
    my ($word, $line, $start) = @_;
 
+   $self->debug && $self->log->debug("comp_run: true");
+
    my $context = $self->context;
 
    my @words = $self->line_parsed($line);
    my $count = scalar(@words);
    my $last = $words[-1];
 
-   $self->debug && $self->log->debug("comp_run: words[@words] | word[$word] line[$line] start[$start] | last[$last]");
+   if ($self->debug) {
+      $self->log->debug("comp_run: words[@words] | word[$word] line[$line] ".
+         "start[$start] | last[$last]");
+   }
 
    # Completion is for used Briks only
    my $used = $context->used;
@@ -911,17 +1246,9 @@ sub comp_run {
          }
       }
 
-      my $brik_commands = Metabrik->brik_properties->{commands};
-      my $commands = $used->{$brik}->brik_commands;
-
-      if (! $self->comp_show_inherited) {
-         $commands = $used->{$brik}->brik_properties->{commands};
-      }
+      my $commands = $self->get_comp_commands($brik);
 
       for my $command (keys %$commands) {
-         if (! $context->get('core::shell', 'comp_show_brik_commands')) {
-            next if exists($brik_commands->{$command});
-         }
          push @comp, $command;
       }
    }
@@ -934,17 +1261,9 @@ sub comp_run {
          }
       }
 
-      my $brik_commands = Metabrik->brik_properties->{commands};
-      my $commands = $used->{$brik}->brik_commands;
-
-      if (! $self->comp_show_inherited) {
-         $commands = $used->{$brik}->brik_properties->{commands};
-      }
+      my $commands = $self->get_comp_commands($brik);
 
       for my $command (keys %$commands) {
-         if (! $context->get('core::shell', 'comp_show_brik_commands')) {
-            next if exists($brik_commands->{$command});
-         }
          if ($command =~ /^$word/) {
             push @comp, $command;
          }
@@ -1033,6 +1352,8 @@ sub catch_comp_sub {
    # Strange, we had to reverse order for $start and $line only for catch_comp() method.
    my ($word, $start, $line) = @_;
 
+   $self->debug && $self->log->debug("catch_comp_sub: true");
+
    my $context = $self->context;
 
    my $attribs = $self->term->Attribs;
@@ -1060,7 +1381,7 @@ sub catch_comp_sub {
 
       for my $this (@$variables) {
          $this =~ s/^\$//;
-         #$self->debug && $self->log->debug("variable[$this] start[$start]");
+         $self->debug && $self->log->debug("variable[$this] start[$start]");
          if ($this =~ /^$word/) {
             push @comp, $this;
          }
@@ -1077,12 +1398,12 @@ sub catch_comp_sub {
          $path = $1 || '/';
       }
 
-      #$self->debug && $self->log->debug("path[$path]");
+      $self->debug && $self->log->debug("path[$path]");
 
       my $found = $self->_file_find($path);
 
       for my $this (@{$found->{files}}, @{$found->{directories}}) {
-         #$self->debug && $self->log->debug("check[$this]");
+         $self->debug && $self->log->debug("check[$this]");
          if ($this =~ /^$word/) {
             push @comp, $this;
          }
@@ -1103,6 +1424,8 @@ sub catch_comp {
    # Strange, we had to reverse order for $start and $line only for catch_comp() method.
    my ($word, $start, $line) = @_;
 
+   $self->debug && $self->log->debug("catch_comp: true");
+
    my $context = $self->context;
 
    my $attribs = $self->term->Attribs;
@@ -1130,12 +1453,17 @@ sub catch_comp {
 
       for my $this (@$variables) {
          $this =~ s/^\$//;
-         #$self->debug && $self->log->debug("variable[$this] start[$start]");
+         $self->debug && $self->log->debug("variable[$this] start[$start]");
          if ($this =~ /^$start/) {
             push @comp, $this;
          }
       }
    }
+   # If we just finished completing an alias, we return the full command
+   elsif (exists($self->{_aliases}{"run_$last"})) {
+      push @comp, $self->{_aliases}{"run_$last"};
+   }
+   # Otherwise, we complete relative filenames
    else {
       my $path = '.';
 
@@ -1151,13 +1479,15 @@ sub catch_comp {
       my $found = $self->_file_find($path);
 
       for my $this (@{$found->{files}}, @{$found->{directories}}) {
-         #$self->debug && $self->log->debug("check[$this]");
+         $self->debug && $self->log->debug("check[$this]");
          if ($this =~ /^$start/) {
             push @comp, $this;
          }
       }
    }
 
+   $self->debug && $self->log->debug("catch_comp: possible [@comp]");
+
    return @comp;
 }
 
