@@ -1,5 +1,5 @@
 #
-# $Id: Command.pm,v ed08d3ef999e 2015/03/16 06:40:47 gomor $
+# $Id: Command.pm,v b5b4517d802d 2015/10/12 19:17:12 gomor $
 #
 # shell::command Brik
 #
@@ -7,37 +7,39 @@ package Metabrik::Shell::Command;
 use strict;
 use warnings;
 
-our $VERSION = '1.08';
+our $VERSION = '1.10';
 
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: ed08d3ef999e $',
+      revision => '$Revision: b5b4517d802d $',
       tags => [ qw(main shell command system) ],
       attributes => {
          as_array => [ qw(0|1) ],
          as_matrix => [ qw(0|1) ],
          capture_stderr => [ qw(0|1) ],
+         capture_mode => [ qw(0|1) ],
+         ignore_error => [ qw(0|1) ],
+         use_sudo => [ qw(0|1) ],
+         sudo_args => [ qw(args) ],
       },
-      commands => {
-         system => [ qw(command) ],
-         capture => [ qw(command) ],
-      },
-      require_modules => {
-         'IPC::Run3' => [ ],
-      },
-   };
-}
-
-sub brik_use_properties {
-   my $self = shift;
-
-   return {
       attributes_default => {
          as_array => 1,
          as_matrix => 0,
          capture_stderr => 1,
+         capture_mode => 0,
+         ignore_error => 1,
+         use_sudo => 0,
+         sudo_args => '-E',  # Keep environment
+      },
+      commands => {
+         system => [ qw(command) ],
+         capture => [ qw(command) ],
+         execute => [ qw(command) ],
+      },
+      require_modules => {
+         'IPC::Run3' => [ ],
       },
    };
 }
@@ -78,11 +80,33 @@ sub system {
    }
 
    $command = join(' ', @toks);
+
+   if ($self->use_sudo) {
+      my @sudo = ( "sudo" );
+      if (! ref($self->sudo_args) && length($self->sudo_args)) {
+         my @args = split(/\s+/, $self->sudo_args);
+         push @sudo, @args;
+      }
+      $command = join(' ', @sudo)." $command";
+      @toks = ( @sudo, @toks );
+      $self->log->verbose("system: using sudo: running [$command]");
+   }
+
    my $r = CORE::system($command);
 
-   # system() returns program exit code in case of success
-   # So we return $? in case of success, and 0 otherwise
-   return defined($r) ? $? : 0;
+   $self->debug && $self->log->debug("system: return code [$r] with status [$?]");
+
+   # Success
+   if ($r == 0) {
+      return 1;
+   }
+
+   if (! $self->ignore_error) {
+      # Failure, we return the program exit code
+      return $?;
+   }
+
+   return 1;
 }
 
 sub capture {
@@ -117,11 +141,29 @@ sub capture {
    $toks[0] = $bin;
 
    if (! -f $bin) {
-      return $self->log->error("system: program [$bin] not found in PATH");
+      return $self->log->error("capture: program [$bin] not found in PATH");
    }
 
+   # Perform file globbing, if any
+   my @globbed = ();
+   for (@toks) {
+      push @globbed, glob($_);
+   }
+   @toks = @globbed;
+
    $command = join(' ', @toks);
 
+   if ($self->use_sudo) {
+      my @sudo = ( "sudo" );
+      if (! ref($self->sudo_args) && length($self->sudo_args)) {
+         my @args = split(/\s+/, $self->sudo_args);
+         push @sudo, @args;
+      }
+      $command = join(' ', @sudo)." $command";
+      @toks = ( @sudo, @toks );
+      $self->log->verbose("capture: using sudo: running [$command]");
+   }
+
    my $out;
    my $err;
    eval {
@@ -135,7 +177,14 @@ sub capture {
    # Error in command execution
    elsif ($?) {
       chomp($err);
-      return $self->log->error("capture: command execution failed [$command]: $err");
+      chomp($out);
+      $err ||= $out; # Sometimes, the error is printed on stdout instead of stderr
+      if ($self->ignore_error) {
+         $self->log->warning("capture: command execution had errors [$command]: $err");
+      }
+      else {
+         return $self->log->error("capture: command execution failed [$command]: $err");
+      }
    }
 
    $out ||= 'undef';
@@ -144,7 +193,7 @@ sub capture {
    chomp($err);
 
    # If we also wanted stderr, we put it at the end of output
-   if ($self->capture_stderr) {
+   if ($self->capture_stderr && $err ne 'undef') {
       $out .= "\n\nSTDERR:\n$err";
    }
 
@@ -163,6 +212,25 @@ sub capture {
    return $out;
 }
 
+sub execute {
+   my $self = shift;
+   my ($cmd, @args) = @_;
+   
+   if (! defined($cmd)) {
+      return $self->log->error($self->brik_help_run('execute'));
+   }
+
+   if ($self->capture_mode) {
+      return $self->capture($cmd, @args);
+   }
+   else {  # non-capture mode
+      return $self->system($cmd, @args);
+   }
+
+   # Unknown error
+   return;
+}
+
 1;
 
 __END__
