@@ -1,5 +1,5 @@
 #
-# $Id: Rc.pm,v d32c8bd9203c 2015/03/09 19:52:04 gomor $
+# $Id: Rc.pm,v 89ee6044675e 2015/10/14 17:16:49 gomor $
 #
 # shell::rc Brik
 #
@@ -7,18 +7,21 @@ package Metabrik::Shell::Rc;
 use strict;
 use warnings;
 
-our $VERSION = '1.08';
+our $VERSION = '1.10';
 
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: d32c8bd9203c $',
+      revision => '$Revision: 89ee6044675e $',
       tags => [ qw(main shell rc) ],
       attributes => {
          file => [ qw(file) ],
          create_default => [ qw(0|1) ],
       },
+      attributes_default => {
+         create_default => 1,
+      },
       commands => {
          load => [ qw(input_file|OPTIONAL) ],
          exec => [ qw($line_list) ],
@@ -33,7 +36,6 @@ sub brik_use_properties {
    return {
       attributes_default => {
          file => $self->global->homedir.'/.metabrik_rc',
-         create_default => 1,
       },
    };
 }
@@ -126,16 +128,15 @@ alias capture "run shell::command capture"
 alias install "run perl::module install"
 alias search "run brik::search"
 alias show "run brik::search all"
-alias ls "capture ls -F"
-alias l "ls -l"
-alias w "capture w"
-alias perldoc "system perldoc"
-alias top "system top"
+alias ls "run shell::command capture ls -F"
+alias l "run shell::command capture ls -lF"
+alias ll "run shell::command capture ls -lF"
+alias w "run shell::command capture w"
 alias history "run shell::history show"
 alias ! "run shell::history exec"
 alias cat "run shell::command capture cat"
 alias pwd "run core::shell pwd"
-alias su "system \$sudo"
+alias su "run shell::command system \$sudo"
 
 use shell::history
 run shell::history load
