@@ -1,5 +1,5 @@
 #
-# $Id: Script.pm,v d32c8bd9203c 2015/03/09 19:52:04 gomor $
+# $Id: Script.pm,v 07f7ae422123 2015/10/09 09:17:45 gomor $
 #
 # shell::script Brik
 #
@@ -7,19 +7,19 @@ package Metabrik::Shell::Script;
 use strict;
 use warnings;
 
-our $VERSION = '1.08';
+our $VERSION = '1.10';
 
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: d32c8bd9203c $',
+      revision => '$Revision: 07f7ae422123 $',
       tags => [ qw(main shell script) ],
       attributes => {
          file => [ qw(file) ],
       },
       attributes_default => {
-         file => 'script.brik',
+         file => 'script.meta',
       },
       commands => {
          load => [ qw(input_file|OPTIONAL) ],
@@ -40,7 +40,7 @@ sub load {
 
    my @lines = ();
    open(my $in, '<', $file)
-         or return $self->log->error("load: can't open file [$file]: $!");
+      or return $self->log->error("load: can't open file [$file]: $!");
    while (defined(my $line = <$in>)) {
       chomp($line);
       push @lines, $line;
